;**************************************************************************************************************************************************
;   PROJECT:      Generatore di Funzioni DSS
;   FILE:         SPI.asm
;   LAST REV:     3.0
;	AUTHOR:       Ivo Colleoni
;  	DEVICE:       PIC18F2550
;	CREATED:	  15/02/2013
;	UPDATED:	  04/04/2013
;
;	DESCRIP:      Simulazione protocollo SPI, invio dati ai relativi componenti
;                 Invia su SPI_D il byte contenuto in spi_data in modo seriale con clock su SPI_CK
;
;   FUNCTION:     SPI_START
;                 SPI_SEND_FREQ
;                 SPI_SEND_PHASE
;                 SPI_SEND_CTRL
;                 SPI_SEND_MUX_VPP
;                 SPI_SEND_POT_OFFS
;                 SPI_SEND (uso interno)
;                 FORMAT_HEX (uso interno)
;
;   PRECONDITION: inserire all'inizio del programma principale:
;                 #DEFINE     SPI_D    LATx,x	per definire dove  collegato il pin Data out
;                 #DEFINE     LCD_CK   LATx,x   per definire dove  collegato il Clock
;                 #DEFINE     FSYNC_DSS LATx,x  per definire dove  collegato il pin di abilitazione del DSS
;--------------------------------------------------------------------------------------------------------------------------------------------------
;   Change History:
;  	Rev.    Date                Changes
;   3.0     04/04/2013          - Implementato invio dato al potenziometro per la regolazione dell'offset
;   2.0     30/03/2013          - Implementato invio dato al potenziometro per la regolazione del Vpp
;   1.0     15/02/2013          - prima build
;
;**************************************************************************************************************************************************
;Dichiarazione variabili Memoria allocata da 0x22 a 0x23
;**************************************************************************************************************************************************
                    CBLOCK              0x22
                    spi_count
                    spi_data
                    ENDC
;
;**************************************************************************************************************************************************
;   FUNCTION:       SPI_START
;   Overview:       porta a 1 i bit di abilitazione  e il clock e porta a 0 il bit data
;**************************************************************************************************************************************************
;
SPI_START           bsf                 FSYNC_DSS                                       ;imposto i valori iniziali SPI
                    bsf                 FSYNC_MUX_VPP
                    bsf                 FSYNC_POT_OFFS
                    bcf					SPI_D
                    bsf                 SPI_CK
                    return
;
;**************************************************************************************************************************************************
;   FUNCTION:       SPI_SEND_FREQ
;   Input:          freq_hex_0, freq_hex_1, freq_hex_2, freq_hex_3
;   Output          spi_data
;   Overview:       dopo aver chiamato il sottoprogramma per la formattazione invia i 4 byte con il valore da impostare in FREQREG al DSS
;                   MSB first
;**************************************************************************************************************************************************
;
SPI_SEND_FREQ       call                FORMAT_HEX                                      ;inserisco nei valori della frequenza il registro FREQ del DSS da usare (sepre 01)

                    bcf                 FSYNC_DSS
                    movff               freq_hex_1, spi_data                            ;mando i 4 byte della frequenza (gi adattati con i 2 bit piu alti di
                    call                SPI_SEND                                        ;freq_hex_1 e freq_hex_3 impostati per scrivere in freq0
                    movff               freq_hex_0, spi_data
                    call                SPI_SEND
                    movff               freq_hex_3, spi_data
                    call                SPI_SEND
                    movff               freq_hex_2, spi_data
                    call                SPI_SEND
                    bsf                 FSYNC_DSS                                       ;disabilito il DSS alla ricezione dati

                    return
;
;**************************************************************************************************************************************************
;   FUNCTION:       SPI_SEND_PHASE
;   Input:          phase_h
;   Output          spi_data
;   Overview:       invia il dato relativo alla fase al DSS
;**************************************************************************************************************************************************
;
SPI_SEND_PHASE      bcf                 FSYNC_DSS
                    movlw               b'11000000'                                     ;D15,D14,D13,D12:1,1 (fase),0 (registro 0), indifferente
                    addwf               phase_h,W                                       ;aggiungo il valore della fase
                    movwf               spi_data
                    call                SPI_SEND
                    movlw               b'00000000'                                     ;gli 8 bit meno significativi della fase saranno sempre 0 per come ho
                    movwf               spi_data                                        ;impostato la gestione della fase
                    call                SPI_SEND
                    bsf                 FSYNC_DSS

                    return
;
;**************************************************************************************************************************************************
;   FUNCTION:       SPI_SEND_CONTROL
;   Input:          dss_ctrl_h, dss_ctrl_l
;   Output          spi_data
;   Overview:       invia al DSS 2 byte che formano la parola a 16 bit del CONTROL REGISTER dove vengono dichiarati i FREQREG e PHASEREG da usare
;                   (in questo caso sempre 0) e la forma d'onda desiderata.
;**************************************************************************************************************************************************
;
SPI_SEND_CTRL       bcf                 FSYNC_DSS                                       ;abilito la ricezione dati
                    movff               dss_ctrl_h,spi_data                             ;mando la parte alta
                    call                SPI_SEND
                    movff               dss_ctrl_l,spi_data                             ;mando la parte bassa
                    call                SPI_SEND
                    bsf                 FSYNC_DSS                                       ;disabilito la ricezione dati

                    return
;
;**************************************************************************************************************************************************
;   FUNCTION:       SPI_SEND_MUX_VPP
;   Input:          mux_vpp
;   Output          spi_data
;   Overview:       invia ai multiplexer il dato contenuto in mux_vpp per abilitare lo switch corretto.
;**************************************************************************************************************************************************
;
SPI_SEND_MUX_VPP    bcf                 FSYNC_MUX_VPP                                   ;abilito i multiplexer alla ricezione dei dati
                    movff               mux_vpp, spi_data                               ;invio il byte del valore
                    call                SPI_SEND
                    bsf                 FSYNC_MUX_VPP                                   ;disabilito i multiplezer alla ricezione dati

                    return
;
;**************************************************************************************************************************************************
;   FUNCTION:       SPI_SEND_POT_OFFS
;   Input:          pot_vpp
;   Output          spi_data
;   Overview:       invia al potenziometro digitale dedicato al controllo dell'offset il byte ci controllo e il dato contenuto in pot_offset
;**************************************************************************************************************************************************
;
SPI_SEND_POT_OFFS   bcf                 FSYNC_POT_OFFS                                  ;abilito il Potenziometro alla ricezione dei dati
                    movlw               b'00010001'                                     ;D0:1 pot.0 D5,D1:01 operazione di scrittura
                    movwf               spi_data                                        ;invio il byte di controlo standard
                    call                SPI_SEND
                    movff               pot_offset, spi_data                            ;invio il byte del valore
                    call                SPI_SEND
                    bsf                 FSYNC_POT_OFFS                                  ;disabilito il Potenziometro alla ricezione dati

                    return
;
;**************************************************************************************************************************************************
;   FUNCTION:       SPI_SEND    (uso interno)
;   Precondition:   chiamata da una delle funzioni precedenti SPI_SEND_xxx
;   Input:          spi_data
;   Overview:       ruota spi_data attraverso il carry per 8 volte, a seconda del dato setta o resetta SPI_D dopodiche il clock viene abbassato e
;                   rialzato MSB first, acquisizione data sul fronte di discesa del clock per il DSS e sul fronte di salita per i potenziometri
;**************************************************************************************************************************************************
;
SPI_SEND            movlw               h'8'                                            ;contatore per gli 8 bit da mandare
                    movwf               spi_count

SPI_CYCLE           rlcf                spi_data                                        ;ruoto il dato e il bit pi alto va nel Carry

                    btfsc               STATUS,C                                        ;se C=1
                    bsf                 SPI_D                                           ;Dato=1
                    btfss               STATUS,C                                        ;se C=0
                    bcf                 SPI_D                                           ;Dato=0

                    nop                                                                 ;nop di simmetria del dato
					nop
;                    nop
;                    nop
;                    nop
;                    nop

                    bcf                 SPI_CK                                          ;abaasso il Clock

                    nop
                    nop
					nop
					nop
;                    nop
;                    nop
;                    nop
;                    nop

                    bsf                 SPI_CK                                          ;riporto alto il Clock

;                    nop
;                    nop
;                    nop
;                    nop
;                    nop
;                    nop
;                    nop
;                    nop

                    decfsz              spi_count                                       ;decremento il contatore, se  0 salto e torno
                    goto                SPI_CYCLE

                    return
;
;**************************************************************************************************************************************************
;   FUNCTION:       FORMAT_HEX  (uso interno)
;   Input:          freq_hex_0, freq_hex_1, freq_hex_2, freq_hex_3
;   Output:         freq_hex_0, freq_hex_1, freq_hex_2, freq_hex_3
;   Overview:       Prepara i 4 byte per l'invio all'AD9833. Questo si aspetta 2 parole da 16 bit l'una dove i 2 bit piu signifivi impostano il registro
;                   da utilizzare.
;                   Spostara i 2 bit piu significativi di freq_hex_1 nei 2 bit meno significativi di freq_hex_2, i 2 bit pi
;                   significativi di freq_hex_2 nei 2 bit meno significativi di freq_hex_3 e quest'ultimo schifta a sinistra di 2 bit.
;                   Aggiunge in freq_hex_1 e freq_hex_3 i 2 bit per la selezione del registro, utilizzo sempre il registro FREQ0 (01)
;**************************************************************************************************************************************************
;
FORMAT_HEX          rlncf               freq_hex_3
                    rlncf               freq_hex_3
                    movlw               b'11111100'                                     ;escludo i 2 bit piu bassi che possono rientrare dallo shift
                    andwf               freq_hex_3,F

                    movf                freq_hex_2,W
                    andlw               b'11000000'                                     ;prendo i 2 bit piu significativi
					movwf				temp

                    rlncf               temp                                            ;sposto temp a sinistra di posizioni per spostare i 2 bit
                    rlncf               temp                                            ;nei 2 bit meno significativi
					movf				temp,w

                    addwf               freq_hex_3,f


                    rlncf               freq_hex_2
                    rlncf               freq_hex_2
                    movlw               b'11111100'                                     ;escludo i 2 bit piu bassi che possono rientrare dallo shift
                    andwf               freq_hex_2,F

                    movf                freq_hex_1,W
                    andlw               b'11000000'                                     ;prendo i 2 bit piu significativi
					movwf				temp

                    rlncf               temp                                            ;sposto temp a sinistra di posizioni per spostare i 2 bit
                    rlncf               temp                                            ;nei 2 bit meno significativi
					movf				temp,w

                    addwf               freq_hex_2,F

                    bsf                 freq_hex_3,6                                    ;imposto il valore del registro a 01
                    bcf                 freq_hex_3,7                                    ;in freq_hex_3 e freq_hex_1

                    bsf                 freq_hex_1,6
                    bcf                 freq_hex_1,7

                    return
;
;**************************************************************************************************************************************************
;   FUNCTION:       RESET_VAR_SPI
;   Overview:       Resetta tutte le variabili utilizzate per la gestione del protocollo SPI
;**************************************************************************************************************************************************
;
RESET_VAR_SPI       clrf                spi_count
                    clrf                spi_data
                    return